@@ -130,3 +130,55 @@ class TestStorage(test.TestCase):
         disk_dvr.disconnect_disk_from_mgmt.assert_called_with('vios_uuid',
                                                               'stg_name')
         mock_rm.assert_called_with('/dev/disk')
+
+    def test_finddisk(self):
+        disk_dvr = mock.Mock()
+        disk_dvr.get_disk_ref.return_value = 'disk_ref'
+        instance = mock.Mock()
+        context = 'context'
+        disk_type = 'disk_type'
+
+        task = tf_stg.FindDisk(disk_dvr, context, instance, disk_type)
+        ret_disk = task.execute()
+        disk_dvr.get_disk_ref.assert_called_once_with(instance, disk_type)
+        self.assertEqual('disk_ref', ret_disk)
+
+        # Bad path for no disk found
+        disk_dvr.reset_mock()
+        disk_dvr.get_disk_ref.return_value = None
+        ret_disk = task.execute()
+        disk_dvr.get_disk_ref.assert_called_once_with(instance, disk_type)
+        self.assertIsNone(ret_disk)
+
+    def test_extenddisk(self):
+        disk_dvr = mock.Mock()
+        instance = mock.Mock()
+        context = 'context'
+        disk_info = {'type': 'disk_type'}
+
+        task = tf_stg.ExtendDisk(disk_dvr, context, instance, disk_info, 1024)
+        task.execute()
+        disk_dvr.extend_disk.assert_called_once_with(context, instance,
+                                                     disk_info, 1024)
+
+    def test_connect_volume(self):
+        vol_dvr = mock.Mock(connection_info={'data': {'volume_id': '1'}})
+
+        task = tf_stg.ConnectVolume(vol_dvr, 'slot map')
+        task.execute()
+        vol_dvr.connect_volume.assert_called_once_with('slot map')
+
+        task.revert('result', 'flow failures')
+        vol_dvr.reset_stg_ftsk.assert_called_once_with()
+        vol_dvr.disconnect_volume.assert_called_once_with('slot map')
+
+    def test_disconnect_volume(self):
+        vol_dvr = mock.Mock(connection_info={'data': {'volume_id': '1'}})
+
+        task = tf_stg.DisconnectVolume(vol_dvr, 'slot map')
+        task.execute()
+        vol_dvr.disconnect_volume.assert_called_once_with('slot map')
+
+        task.revert('result', 'flow failures')
+        vol_dvr.reset_stg_ftsk.assert_called_once_with()
+        vol_dvr.connect_volume.assert_called_once_with('slot map')
