@@ -259,12 +259,41 @@ class TestVM(test.TestCase):
             vm._uuid_set_high_bit('f6f79d3f-eef1-4009-bfd4-172ab7e6fff4'),
             'f6f79d3f-eef1-4009-bfd4-172ab7e6fff4')
 
+    def test_translate_vm_state(self):
+        self.assertEqual(power_state.RUNNING,
+                         vm._translate_vm_state('running'))
+        self.assertEqual(power_state.RUNNING,
+                         vm._translate_vm_state('migrating running'))
+        self.assertEqual(power_state.RUNNING,
+                         vm._translate_vm_state('starting'))
+        self.assertEqual(power_state.RUNNING,
+                         vm._translate_vm_state('open firmware'))
+        self.assertEqual(power_state.RUNNING,
+                         vm._translate_vm_state('shutting down'))
+        self.assertEqual(power_state.RUNNING,
+                         vm._translate_vm_state('suspending'))
+
+        self.assertEqual(power_state.SHUTDOWN,
+                         vm._translate_vm_state('migrating not active'))
+        self.assertEqual(power_state.SHUTDOWN,
+                         vm._translate_vm_state('not activated'))
+
+        self.assertEqual(power_state.NOSTATE,
+                         vm._translate_vm_state('unknown'))
+        self.assertEqual(power_state.NOSTATE,
+                         vm._translate_vm_state('hardware discovery'))
+        self.assertEqual(power_state.NOSTATE,
+                         vm._translate_vm_state('not available'))
+
+        self.assertEqual(power_state.SUSPENDED,
+                         vm._translate_vm_state('resuming'))
+        self.assertEqual(power_state.SUSPENDED,
+                         vm._translate_vm_state('suspended'))
+
+        self.assertEqual(power_state.CRASHED,
+                         vm._translate_vm_state('error'))
+
     def test_instance_info(self):
-
-        # Test at least one state translation
-        self.assertEqual(vm._translate_vm_state('running'),
-                         power_state.RUNNING)
-
         inst_info = vm.InstanceInfo(self.apt, 'inst_name', '1234')
         # Test the static properties
         self.assertEqual(inst_info.id, '1234')
@@ -415,16 +444,20 @@ class TestVM(test.TestCase):
         self.assertEqual('NewEntry', new_entry)
         self.san_lpar_name.assert_called_with(name)
 
+    @mock.patch('pypowervm.utils.transaction.entry_transaction')
     @mock.patch('nova_powervm.virt.powervm.vm.get_instance_wrapper')
-    def test_rename(self, mock_get_inst):
+    def test_rename(self, mock_get_inst, mock_entry_transaction):
         instance = objects.Instance(**powervm.TEST_INSTANCE)
 
+        mock_entry_transaction.side_effect = lambda x: x
+
         entry = mock.Mock()
         entry.update.return_value = 'NewEntry'
         new_entry = vm.rename(self.apt, 'mock_host_uuid', instance, 'new_name',
                               entry=entry)
         self.assertEqual('new_name', entry.name)
         entry.update.assert_called_once_with()
+        mock_entry_transaction.assert_called_once_with(mock.ANY)
         self.assertEqual('NewEntry', new_entry)
         self.san_lpar_name.assert_called_with('new_name')
         self.san_lpar_name.reset_mock()
@@ -460,23 +493,27 @@ class TestVM(test.TestCase):
 
     @mock.patch('pypowervm.tasks.power.power_off')
     def test_power_off(self, mock_power_off):
+        instance = objects.Instance(**powervm.TEST_INSTANCE)
+
         self.assertFalse(vm.power_off(
-            None, None, 'host_uuid',
+            None, instance, 'host_uuid',
             mock.Mock(state=pvm_bp.LPARState.NOT_ACTIVATED)))
         self.assertFalse(mock_power_off.called)
 
-        stop_states = [pvm_bp.LPARState.RUNNING, pvm_bp.LPARState.STARTING,
-                       pvm_bp.LPARState.OPEN_FIRMWARE, pvm_bp.LPARState.ERROR,
-                       pvm_bp.LPARState.RESUMING]
+        stop_states = [
+            pvm_bp.LPARState.RUNNING, pvm_bp.LPARState.STARTING,
+            pvm_bp.LPARState.OPEN_FIRMWARE, pvm_bp.LPARState.SHUTTING_DOWN,
+            pvm_bp.LPARState.ERROR, pvm_bp.LPARState.RESUMING,
+            pvm_bp.LPARState.SUSPENDING]
         for stop_state in stop_states:
             entry = mock.Mock(state=stop_state)
             mock_power_off.reset_mock()
-            self.assertTrue(vm.power_off(None, None, 'host_uuid', entry))
+            self.assertTrue(vm.power_off(None, instance, 'host_uuid', entry))
             mock_power_off.assert_called_once_with(entry, 'host_uuid',
                                                    force_immediate=False,
                                                    add_parms=None)
             mock_power_off.reset_mock()
-            self.assertTrue(vm.power_off(None, None, 'host_uuid', entry,
+            self.assertTrue(vm.power_off(None, instance, 'host_uuid', entry,
                                          force_immediate=True))
             mock_power_off.assert_called_once_with(entry, 'host_uuid',
                                                    force_immediate=True,
@@ -485,13 +522,14 @@ class TestVM(test.TestCase):
     @mock.patch('pypowervm.tasks.power.power_off')
     def test_power_off_negative(self, mock_power_off):
         """Negative tests."""
+        instance = objects.Instance(**powervm.TEST_INSTANCE)
 
         # Raise the expected pypowervm exception
         mock_power_off.side_effect = pvm_exc.VMPowerOffFailure(
             reason='Something bad.', lpar_nm='TheLPAR')
         # We should get a valid Nova exception that the compute manager expects
         self.assertRaises(exception.InstancePowerOffFailure,
-                          vm.power_off, None, None, 'host_uuid',
+                          vm.power_off, None, instance, 'host_uuid',
                           mock.Mock(state=pvm_bp.LPARState.RUNNING))
 
     @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
