@@ -41,7 +41,8 @@ class Get(task.Task):
         :param host_uuid: The host UUID
         :param instance: The nova instance.
         """
-        super(Get, self).__init__(name='get_lpar', provides='lpar_wrap')
+        super(Get, self).__init__(name='get_lpar',
+                                  provides='lpar_wrap')
         self.adapter = adapter
         self.host_uuid = host_uuid
         self.instance = instance
@@ -78,8 +79,7 @@ class Create(task.Task):
         :param nvram_mgr: The NVRAM manager to fetch the NVRAM from. If None,
                           the NVRAM will not be fetched.
         """
-        super(Create, self).__init__(name='crt_lpar',
-                                     provides='lpar_wrap')
+        super(Create, self).__init__(name='crt_lpar', provides='lpar_wrap')
         self.adapter = adapter
         self.host_wrapper = host_wrapper
         self.instance = instance
@@ -94,16 +94,8 @@ class Create(task.Task):
         if self.nvram_mgr is not None:
             LOG.info(_LI('Fetching NVRAM for instance %s.'),
                      self.instance.name, instance=self.instance)
-            try:
-                data = self.nvram_mgr.fetch(self.instance)
-                LOG.debug('NVRAM data is: %s', data, instance=self.instance)
-            except Exception as e:
-                # Fetching NVRAM exception should not cause a failure
-                LOG.exception(_LE('Unable to fetch NVRAM for instance '
-                                  '%(name)s. Exception: %(reason)s'),
-                              {'name': self.instance.name,
-                               'reason': six.text_type(e)},
-                              instance=self.instance)
+            data = self.nvram_mgr.fetch(self.instance)
+            LOG.debug('NVRAM data is: %s', data, instance=self.instance)
 
         wrap = vm.crt_lpar(self.adapter, self.host_wrapper, self.instance,
                            self.flavor, nvram=data)
