@@ -1,4 +1,4 @@
-# Copyright 2015 IBM Corp.
+# Copyright 2015, 2016 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -14,16 +14,19 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
 from oslo_concurrency import lockutils
-from oslo_config import cfg
 from oslo_log import log as logging
 from taskflow import task
 
 from nova.compute import task_states
+from oslo_serialization import jsonutils
+from pypowervm import const as pvm_const
+from pypowervm.tasks import client_storage as pvm_c_stor
 from pypowervm.tasks import vfc_mapper as pvm_vfcm
-from pypowervm.wrappers import virtual_io_server as pvm_vios
 
-from nova_powervm.virt import powervm
+from nova_powervm import conf as cfg
+from nova_powervm.conf import powervm as pvm_cfg
 from nova_powervm.virt.powervm import exception as exc
 from nova_powervm.virt.powervm.i18n import _
 from nova_powervm.virt.powervm.i18n import _LE
@@ -59,22 +62,38 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         """List of pypowervm XAGs needed to support this adapter."""
         # Storage are so physical FC ports are available
         # FC mapping is for the connections between VIOS and client VM
-        return [pvm_vios.VIOS.xags.FC_MAPPING, pvm_vios.VIOS.xags.STORAGE]
+        return [pvm_const.XAG.VIO_FMAP, pvm_const.XAG.VIO_STOR]
 
-    def _connect_volume(self):
-        """Connects the volume."""
+    @classmethod
+    def vol_type(cls):
+        """The type of volume supported by this type."""
+        return 'npiv'
+
+    def _connect_volume(self, slot_mgr):
+        """Connects the volume.
+
+        :param slot_mgr: A NovaSlotManager.  Used to store/retrieve the client
+                         slots used when a volume is attached to the VM
+        """
         # Run the add for each fabric.
         for fabric in self._fabric_names():
-            self._add_maps_for_fabric(fabric)
+            self._add_maps_for_fabric(fabric, slot_mgr)
 
-    def _disconnect_volume(self):
-        """Disconnect the volume."""
+    def _disconnect_volume(self, slot_mgr):
+        """Disconnect the volume.
+
+        :param slot_mgr: A NovaSlotManager.  Used to delete the client slots
+                         used when a volume is detached from the VM
+        """
         # We should only delete the NPIV mappings if we are running through a
         # VM deletion.  VM deletion occurs when the task state is deleting.
         # However, it can also occur during a 'roll-back' of the spawn.
         # Disconnect of the volumes will only be called during a roll back
-        # of the spawn.
-        if self.instance.task_state not in TASK_STATES_FOR_DISCONNECT:
+        # of the spawn. We also want to check that the instance is on this
+        # host. If it isn't then we can remove the mappings because this is
+        # being called as the result of an evacuation clean up.
+        if (self.instance.task_state not in TASK_STATES_FOR_DISCONNECT and
+           self.instance.host in [None, CONF.host]):
             # NPIV should only remove the VFC mapping upon a destroy of the VM
             return
 
@@ -116,10 +135,11 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
                 client_slots.append(vfc_map.client_adapter.lpar_slot_num)
 
             # Set the client slots into the fabric data to pass to the
-            # destination.
-            mig_data['npiv_fabric_slots_%s' % fabric] = client_slots
+            # destination. Only strings can be stored.
+            mig_data['src_npiv_fabric_slots_%s' % fabric] = (
+                jsonutils.dumps(client_slots))
 
-    def pre_live_migration_on_destination(self, src_mig_data, dest_mig_data):
+    def pre_live_migration_on_destination(self, mig_data):
         """Perform pre live migration steps for the volume on the target host.
 
         This method performs any pre live migration that is needed.
@@ -132,11 +152,10 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         method.  The data from the pre_live call will be passed in via the
         mig_data.  This method should put its output into the dest_mig_data.
 
-        :param src_mig_data: The migration data from the source server.
-        :param dest_mig_data: The migration data for the destination server.
-                              If the volume connector needs to provide
-                              information to the live_migration command, it
-                              should be added to this dictionary.
+        :param mig_data: Dict of migration data for the destination server.
+                         If the volume connector needs to provide
+                         information to the live_migration command, it
+                         should be added to this dictionary.
         """
         vios_wraps = self.stg_ftsk.feed
 
@@ -145,20 +164,22 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         # the source system what 'vfc mappings' to pass in on the live
         # migration command.
         for fabric in self._fabric_names():
-            slots = src_mig_data['npiv_fabric_slots_%s' % fabric]
+            slots = jsonutils.loads(
+                mig_data['src_npiv_fabric_slots_%s' % fabric])
             fabric_mapping = pvm_vfcm.build_migration_mappings_for_fabric(
                 vios_wraps, self._fabric_ports(fabric), slots)
-            dest_mig_data['npiv_fabric_mapping_%s' % fabric] = fabric_mapping
+            mig_data['dest_npiv_fabric_mapping_%s' % fabric] = (
+                jsonutils.dumps(fabric_mapping))
             # Reverse the vios wrapper so that the other fabric will get the
             # on the second vios.
             vios_wraps.reverse()
 
         # Collate all of the individual fabric mappings into a single element.
         full_map = []
-        for key, value in dest_mig_data.items():
-            if key.startswith('npiv_fabric_mapping_'):
-                full_map.extend(value)
-        dest_mig_data['vfc_lpm_mappings'] = full_map
+        for key, value in mig_data.items():
+            if key.startswith('dest_npiv_fabric_mapping_'):
+                full_map.extend(jsonutils.loads(value))
+        mig_data['vfc_lpm_mappings'] = jsonutils.dumps(full_map)
 
     def post_live_migration_at_destination(self, mig_vol_stor):
         """Perform post live migration steps for the volume on the target host.
@@ -178,6 +199,8 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         # manager.  Given that, we need to update the order of the WWPNs.
         # The first WWPN is the one that is logged into the fabric and this
         # will now indicate that our WWPN is logged in.
+        LOG.debug('Post live migrate volume store: %s' % mig_vol_stor,
+                  instance=self.instance)
         for fabric in self._fabric_names():
             # We check the mig_vol_stor to see if this fabric has already been
             # flipped.  If so, we can continue.
@@ -192,7 +215,8 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
                 # Flip the WPWNs
                 c_wwpns = port_map[1].split()
                 c_wwpns.reverse()
-
+                LOG.debug('Flipping WWPNs, ports: %s wwpns: %s' %
+                          (port_map, c_wwpns), instance=self.instance)
                 # Get the new physical WWPN.
                 vfc_map = pvm_vfcm.find_vios_for_vfc_wwpns(vios_wraps,
                                                            c_wwpns)[1]
@@ -385,19 +409,22 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         # The return object needs to be a list for the volume connector.
         return resp_wwpns
 
-    def _add_maps_for_fabric(self, fabric):
+    def _add_maps_for_fabric(self, fabric, slot_mgr):
         """Adds the vFC storage mappings to the VM for a given fabric.
 
         :param fabric: The fabric to add the mappings to.
+        :param slot_mgr: A NovaSlotManager.  Used to store/retrieve the client
+                         slots used when a volume is attached to the VM
         """
         npiv_port_maps = self._get_fabric_meta(fabric)
         vios_wraps = self.stg_ftsk.feed
         volume_id = self.connection_info['data']['volume_id']
 
         # This loop adds the maps from the appropriate VIOS to the client VM
+        slot_ids = copy.deepcopy(slot_mgr.build_map.get_vfc_slots(
+            fabric, len(npiv_port_maps)))
         for npiv_port_map in npiv_port_maps:
             vios_w = pvm_vfcm.find_vios_for_port_map(vios_wraps, npiv_port_map)
-
             if vios_w is None:
                 LOG.error(_LE("Mappings were not able to find a proper VIOS. "
                               "The port mappings were %s."), npiv_port_maps)
@@ -405,15 +432,40 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
                     volume_id=volume_id, instance_name=self.instance.name,
                     reason=_("Unable to find a Virtual I/O Server that "
                              "hosts the NPIV port map for the server."))
-
             ls = [LOG.info, _LI("Adding NPIV mapping for instance %(inst)s "
                                 "for Virtual I/O Server %(vios)s."),
                   {'inst': self.instance.name, 'vios': vios_w.name}]
 
             # Add the subtask to add the specific map.
+            slot_num = slot_ids.pop()
             self.stg_ftsk.wrapper_tasks[vios_w.uuid].add_functor_subtask(
                 pvm_vfcm.add_map, self.host_uuid, self.vm_uuid, npiv_port_map,
-                logspec=ls)
+                lpar_slot_num=slot_num, logspec=ls)
+
+        # Store the client slot number for the NPIV mapping (for rebuild
+        # scenarios)
+        def set_vol_meta():
+            vios_wraps = self.stg_ftsk.feed
+            port_maps = self._get_fabric_meta(fabric)
+            for port_map in port_maps:
+                # The port map is [ 'phys_wwpn', 'client_wwpn1 client_wwpn2' ]
+                # We only need one of the two client wwpns.
+                vios_w = pvm_vfcm.find_vios_for_port_map(vios_wraps, port_map)
+                c_wwpns = port_map[1].split()
+                vfc_mapping = pvm_c_stor.c_wwpn_to_vfc_mapping(vios_w,
+                                                               c_wwpns[0])
+
+                # If there is no mapping, then don't add it.  It means that
+                # the client WWPN is hosted on a different VIOS.
+                if vfc_mapping is None:
+                    continue
+
+                # However, by this point we know that it is hosted on this
+                # VIOS.  So the vfc_mapping will have the client adapter
+                slot_mgr.register_vfc_mapping(vfc_mapping, fabric)
+
+        self.stg_ftsk.add_post_execute(task.FunctorTask(
+            set_vol_meta, name='fab_slot_%s_%s' % (fabric, volume_id)))
 
         # After all the mappings, make sure the fabric state is updated.
         def set_state():
@@ -446,9 +498,9 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
                     pvm_vfcm.remove_maps, self.vm_uuid,
                     port_map=npiv_port_map, logspec=ls)
             else:
-                LOG.warn(_LW("No storage connections found between the "
-                             "Virtual I/O Servers and FC Fabric %(fabric)s."),
-                         {'fabric': fabric})
+                LOG.warning(_LW("No storage connections found between the "
+                                "Virtual I/O Servers and FC Fabric "
+                                "%(fabric)s."), {'fabric': fabric})
 
     def host_name(self):
         """Derives the host name that should be used for the storage device.
@@ -460,6 +512,7 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
 
     def _set_fabric_state(self, fabric, state):
         """Sets the fabric state into the instance's system metadata.
+
         :param fabric: The name of the fabric
         :param state: state of the fabric which needs to be set
 
@@ -563,11 +616,11 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
 
         if self.instance.system_metadata.get(meta_key) is None:
             # If no mappings exist, log a warning.
-            LOG.warn(_LW("No NPIV mappings exist for instance %(inst)s on "
-                         "fabric %(fabric)s.  May not have connected to "
-                         "the fabric yet or fabric configuration was recently "
-                         "modified."),
-                     {'inst': self.instance.name, 'fabric': fabric})
+            LOG.warning(_LW("No NPIV mappings exist for instance %(inst)s on "
+                            "fabric %(fabric)s.  May not have connected to "
+                            "the fabric yet or fabric configuration was "
+                            "recently modified."),
+                        {'inst': self.instance.name, 'fabric': fabric})
             return []
 
         wwpns = self.instance.system_metadata[meta_key]
@@ -593,15 +646,14 @@ class NPIVVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
 
     def _fabric_names(self):
         """Returns a list of the fabric names."""
-        return powervm.NPIV_FABRIC_WWPNS.keys()
+        return pvm_cfg.NPIV_FABRIC_WWPNS.keys()
 
     def _fabric_ports(self, fabric_name):
         """Returns a list of WWPNs for the fabric's physical ports."""
-        return powervm.NPIV_FABRIC_WWPNS[fabric_name]
+        return pvm_cfg.NPIV_FABRIC_WWPNS[fabric_name]
 
     def _ports_per_fabric(self):
-        """Returns the number of virtual ports that should be used per fabric.
-        """
+        """Returns the number of virtual ports to be used per fabric."""
         return CONF.powervm.ports_per_fabric
 
     def _get_num_keys(self, port_map):
