@@ -1,4 +1,4 @@
-# Copyright 2015 IBM Corp.
+# Copyright 2015, 2016 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -15,10 +15,10 @@
 #    under the License.
 
 from oslo_concurrency import lockutils
-from oslo_config import cfg
 from oslo_log import log as logging
 from taskflow import task
 
+from nova_powervm import conf as cfg
 from nova_powervm.virt.powervm import exception as p_exc
 from nova_powervm.virt.powervm.i18n import _
 from nova_powervm.virt.powervm.i18n import _LE
@@ -28,6 +28,8 @@ from nova_powervm.virt.powervm import vios
 from nova_powervm.virt.powervm import vm
 from nova_powervm.virt.powervm.volume import driver as v_driver
 
+from pypowervm import const as pvm_const
+from pypowervm.tasks import client_storage as pvm_c_stor
 from pypowervm.tasks import hdisk
 from pypowervm.tasks import scsi_mapper as tsk_map
 from pypowervm.utils import transaction as tx
@@ -76,9 +78,14 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
     def min_xags(cls):
         """List of pypowervm XAGs needed to support this adapter."""
         # SCSI mapping is for the connections between VIOS and client VM
-        return [pvm_vios.VIOS.xags.SCSI_MAPPING]
+        return [pvm_const.XAG.VIO_SMAP]
 
-    def pre_live_migration_on_destination(self, src_mig_data, dest_mig_data):
+    @classmethod
+    def vol_type(cls):
+        """The type of volume supported by this type."""
+        return 'vscsi'
+
+    def pre_live_migration_on_destination(self, mig_data):
         """Perform pre live migration steps for the volume on the target host.
 
         This method performs any pre live migration that is needed.
@@ -91,11 +98,10 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         method.  The data from the pre_live call will be passed in via the
         mig_data.  This method should put its output into the dest_mig_data.
 
-        :param src_mig_data: The migration data from the source server.
-        :param dest_mig_data: The migration data for the destination server.
-                              If the volume connector needs to provide
-                              information to the live_migration command, it
-                              should be added to this dictionary.
+        :param mig_data: Dict of migration data for the destination server.
+                         If the volume connector needs to provide
+                         information to the live_migration command, it
+                         should be added to this dictionary.
         """
         volume_id = self.volume_id
         found = False
@@ -103,7 +109,7 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         # See the connect_volume for why this is a direct call instead of
         # using the tx_mgr.feed
         vios_feed = self.adapter.read(pvm_vios.VIOS.schema_type,
-                                      xag=[pvm_vios.VIOS.xags.STORAGE])
+                                      xag=[pvm_const.XAG.VIO_STOR])
         vios_wraps = pvm_vios.VIOS.wrap(vios_feed)
 
         # Iterate through host vios list to find valid hdisks.
@@ -118,14 +124,14 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
                            instance_name=self.instance.name)
             raise p_exc.VolumePreMigrationFailed(**ex_args)
 
-        dest_mig_data['vscsi-' + volume_id] = udid
+        mig_data['vscsi-' + volume_id] = udid
 
     def _cleanup_volume(self, udid):
         """Cleanup the hdisk associated with this udid."""
 
         if not udid:
-            LOG.warn(_LW('Could not remove hdisk for volume: %s')
-                     % self.volume_id)
+            LOG.warning(_LW('Could not remove hdisk for volume: %s')
+                        % self.volume_id)
             return
 
         LOG.info(_LI('Removing hdisk for udid: %s') % udid)
@@ -142,7 +148,7 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         # Create a feed task to get the vios, find the hdisk and remove it.
         rmv_hdisk_ftsk = tx.FeedTask(
             'find_hdisk_to_remove', pvm_vios.VIOS.getter(
-                self.adapter, xag=[pvm_vios.VIOS.xags.STORAGE]))
+                self.adapter, xag=[pvm_const.XAG.VIO_STOR]))
         # Find vios hdisks for this udid to remove.
         rmv_hdisk_ftsk.add_functor_subtask(
             find_hdisk_to_remove, flag_update=False)
@@ -154,14 +160,13 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         This method can be used to handle any steps that need to taken on
         the source host after the VM is on the destination.
 
-        :param migrate_data: migration data
+        :param migrate_data: volume migration data
         """
         # Get the udid of the volume to remove the hdisk for.  We can't
         # use the connection information because LPM 'refreshes' it, which
         # wipes out our data, so we use the data from the destination host
         # to avoid having to discover the hdisk to get the udid.
-        udid = migrate_data['pre_live_migration_result'].get(
-            'vscsi-' + self.volume_id)
+        udid = migrate_data.get('vscsi-' + self.volume_id)
         self._cleanup_volume(udid)
 
     def cleanup_volume_at_destination(self, migrate_data):
@@ -175,6 +180,18 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         udid = migrate_data.get('vscsi-' + self.volume_id)
         self._cleanup_volume(udid)
 
+    def is_volume_on_vios(self, vios_w):
+        """Returns whether or not the volume is on a VIOS.
+
+        :param vios_w: The Virtual I/O Server wrapper.
+        :return: True if the volume driver's volume is on the VIOS.  False
+                 otherwise.
+        :return: The udid of the device.
+        """
+        status, device_name, udid = self._discover_volume_on_vios(
+            vios_w, self.volume_id)
+        return hdisk.good_discovery(status, device_name), udid
+
     def _discover_volume_on_vios(self, vios_w, volume_id):
         """Discovers an hdisk on a single vios for the volume.
 
@@ -204,15 +221,19 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
                      {'hdisk': device_name, 'vios': vios_w.name,
                       'volume_id': volume_id, 'status': str(status)})
         elif status == hdisk.LUAStatus.DEVICE_IN_USE:
-            LOG.warn(_LW('Discovered device %(dev)s for volume %(volume)s '
-                         'on %(vios)s is in use. Error code: %(status)s.'),
-                     {'dev': device_name, 'volume': volume_id,
-                      'vios': vios_w.name, 'status': str(status)})
+            LOG.warning(_LW('Discovered device %(dev)s for volume %(volume)s '
+                            'on %(vios)s is in use. Error code: %(status)s.'),
+                        {'dev': device_name, 'volume': volume_id,
+                         'vios': vios_w.name, 'status': str(status)})
 
         return status, device_name, udid
 
-    def _connect_volume(self):
-        """Connects the volume."""
+    def _connect_volume(self, slot_mgr):
+        """Connects the volume.
+
+        :param slot_mgr: A NovaSlotManager.  Used to store/retrieve the client
+                         slots used when a volume is attached to the VM
+        """
 
         def connect_volume_to_vio(vios_w):
             """Attempts to connect a volume to a given VIO.
@@ -225,11 +246,15 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
             status, device_name, udid = self._discover_volume_on_vios(
                 vios_w, self.volume_id)
 
+            # Get the slot and LUA to assign.
+            slot, lua = slot_mgr.build_map.get_vscsi_slot(vios_w, udid)
+
             if hdisk.good_discovery(status, device_name):
                 # Found a hdisk on this Virtual I/O Server.  Add the action to
                 # map it to the VM when the stg_ftsk is executed.
                 with lockutils.lock(hash(self)):
-                    self._add_append_mapping(vios_w.uuid, device_name)
+                    self._add_append_mapping(vios_w.uuid, device_name,
+                                             lpar_slot_num=slot, lua=lua)
 
                 # Save the UDID for the disk in the connection info.  It is
                 # used for the detach.
@@ -253,10 +278,13 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         # the stg_ftsk from potentially having to run it multiple times.
         connect_ftsk = tx.FeedTask(
             'connect_volume_to_vio', pvm_vios.VIOS.getter(
-                self.adapter, xag=[pvm_vios.VIOS.xags.STORAGE]))
+                self.adapter, xag=[pvm_const.XAG.VIO_STOR,
+                                   pvm_const.XAG.VIO_SMAP]))
+
         # Find valid hdisks and map to VM.
         connect_ftsk.add_functor_subtask(
             connect_volume_to_vio, provides='vio_modified', flag_update=False)
+
         ret = connect_ftsk.execute()
 
         # Check the number of VIOSes
@@ -264,7 +292,22 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         for result in ret['wrapper_task_rets'].values():
             if result['vio_modified']:
                 vioses_modified += 1
+
+        partition_id = vm.get_vm_id(self.adapter, self.vm_uuid)
+
+        # Update the slot information
+        def set_slot_info():
+            vios_wraps = self.stg_ftsk.feed
+            for vios_w in vios_wraps:
+                scsi_map = pvm_c_stor.udid_to_scsi_mapping(
+                    vios_w, self._get_udid(), partition_id)
+                if not scsi_map:
+                    continue
+                slot_mgr.register_vscsi_mapping(scsi_map)
+
         self._validate_vios_on_connection(vioses_modified)
+        self.stg_ftsk.add_post_execute(task.FunctorTask(
+            set_slot_info, name='hdisk_slot_%s' % self._get_udid()))
 
     def _validate_vios_on_connection(self, num_vioses_found):
         """Validates that the correct number of VIOSes were discovered.
@@ -298,8 +341,12 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
                    'instance_name': self.instance.name}
         raise p_exc.VolumeAttachFailed(**ex_args)
 
-    def _disconnect_volume(self):
-        """Disconnect the volume."""
+    def _disconnect_volume(self, slot_mgr):
+        """Disconnect the volume.
+
+        :param slot_mgr: A NovaSlotManager.  Used to delete the client slots
+                         used when a volume is detached from the VM
+        """
         def discon_vol_for_vio(vios_w):
             """Removes the volume from a specific Virtual I/O Server.
 
@@ -322,7 +369,7 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
                     # in the I/O Server.  Subsequent scrub code on future
                     # deploys will clean this up.
                     if not hdisk.good_discovery(status, device_name):
-                        LOG.warn(_LW(
+                        LOG.warning(_LW(
                             "Disconnect Volume: The backing hdisk for volume "
                             "%(volume_id)s on Virtual I/O Server %(vios)s is "
                             "not in a valid state.  No disconnect "
@@ -334,7 +381,7 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
                     device_name = vios_w.hdisk_from_uuid(udid)
 
                 if not device_name:
-                    LOG.warn(_LW(
+                    LOG.warning(_LW(
                         "Disconnect Volume: No mapped device found on Virtual "
                         "I/O Server %(vios)s for volume %(volume_id)s.  "
                         "Volume UDID: %(volume_uid)s"),
@@ -343,7 +390,7 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
                     return False
 
             except Exception as e:
-                LOG.warn(_LW(
+                LOG.warning(_LW(
                     "Disconnect Volume: Failed to find disk on Virtual I/O "
                     "Server %(vios_name)s for volume %(volume_id)s. Volume "
                     "UDID: %(volume_uid)s.  Error: %(error)s"),
@@ -363,10 +410,9 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
 
             with lockutils.lock(hash(self)):
                 self._add_remove_mapping(partition_id, vios_w.uuid,
-                                         device_name)
+                                         device_name, slot_mgr)
 
-                # Add a step after the mapping removal to also remove the
-                # hdisk.
+                # Add a step to also remove the hdisk
                 self._add_remove_hdisk(vios_w, device_name)
 
             # Found a valid element to remove
@@ -376,7 +422,7 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
             # See logic in _connect_volume for why this new FeedTask is here.
             discon_ftsk = tx.FeedTask(
                 'discon_volume_from_vio', pvm_vios.VIOS.getter(
-                    self.adapter, xag=[pvm_vios.VIOS.xags.STORAGE]))
+                    self.adapter, xag=[pvm_const.XAG.VIO_STOR]))
             # Find hdisks to disconnect
             discon_ftsk.add_functor_subtask(
                 discon_vol_for_vio, provides='vio_modified', flag_update=False)
@@ -385,11 +431,11 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
             # Warn if no hdisks disconnected.
             if not any([result['vio_modified']
                         for result in ret['wrapper_task_rets'].values()]):
-                LOG.warn(_LW("Disconnect Volume: Failed to disconnect the "
-                             "volume %(volume_id)s on ANY of the Virtual I/O "
-                             "Servers for instance %(inst)s."),
-                         {'inst': self.instance.name,
-                          'volume_id': self.volume_id})
+                LOG.warning(_LW("Disconnect Volume: Failed to disconnect the "
+                                "volume %(volume_id)s on ANY of the Virtual "
+                                "I/O Servers for instance %(inst)s."),
+                            {'inst': self.instance.name,
+                             'volume_id': self.volume_id})
 
         except Exception as e:
             LOG.error(_LE('Cannot detach volumes from virtual machine: %s'),
@@ -399,6 +445,27 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
                        'instance_name': self.instance.name}
             raise p_exc.VolumeDetachFailed(**ex_args)
 
+    def _check_host_mappings(self, vios_wrap, device_name):
+        """Checks if the given hdisk has multiple mappings
+
+        :param vio_wrap: The Virtual I/O Server wrapper to remove the disk
+                         from.
+        :param device_name: The hdisk name to remove.
+
+        :return: True is there are multiple instances using the given hdisk
+        """
+        vios_scsi_mappings = next(v.scsi_mappings for v in self.stg_ftsk.feed
+                                  if v.uuid == vios_wrap.uuid)
+        mappings = tsk_map.find_maps(
+            vios_scsi_mappings, None,
+            tsk_map.gen_match_func(pvm_stor.PV, names=[device_name]))
+
+        LOG.info(_LI("%(num)d Storage Mappings found for %(dev)s"),
+                 {'num': len(mappings), 'dev': device_name})
+        # the mapping is still present as the task feed removes
+        # the mapping later
+        return len(mappings) > 1
+
     def _add_remove_hdisk(self, vio_wrap, device_name,
                           stg_ftsk=None):
         """Adds a post-mapping task to remove the hdisk from the VIOS.
@@ -420,13 +487,19 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
                                    vio_wrap.uuid)
             except Exception as e:
                 # If there is a failure, log it, but don't stop the process
-                LOG.warn(_LW("There was an error removing the hdisk "
-                         "%(disk)s from the Virtual I/O Server."),
-                         {'disk': device_name})
-                LOG.warn(e)
-        name = 'rm_hdisk_%s_%s' % (vio_wrap.name, device_name)
-        stg_ftsk = stg_ftsk or self.stg_ftsk
-        stg_ftsk.add_post_execute(task.FunctorTask(rm_hdisk, name=name))
+                LOG.warning(_LW("There was an error removing the hdisk "
+                            "%(disk)s from the Virtual I/O Server."),
+                            {'disk': device_name})
+                LOG.warning(e)
+
+        # Check if there are not multiple mapping for the device
+        if not self._check_host_mappings(vio_wrap, device_name):
+            name = 'rm_hdisk_%s_%s' % (vio_wrap.name, device_name)
+            stg_ftsk = stg_ftsk or self.stg_ftsk
+            stg_ftsk.add_post_execute(task.FunctorTask(rm_hdisk, name=name))
+        else:
+            LOG.info(_LI("hdisk %(disk)s is not removed because it has "
+                         "existing storage mappings"), {'disk': device_name})
 
     @lockutils.synchronized('vscsi_wwpns')
     def wwpns(self):
@@ -448,34 +521,49 @@ class VscsiVolumeAdapter(v_driver.FibreChannelVolumeAdapter):
         """
         return CONF.host
 
-    def _add_remove_mapping(self, vm_uuid, vios_uuid, device_name):
+    def _add_remove_mapping(self, vm_uuid, vios_uuid, device_name, slot_mgr):
         """Adds a transaction to remove the storage mapping.
 
         :param vm_uuid: The UUID of the VM instance
         :param vios_uuid: The UUID of the vios for the pypowervm adapter.
         :param device_name: The The hdisk device name.
+        :param slot_mgr: A NovaSlotManager.  Used to delete the client slots
+                         used when a volume is detached from the VM.
         """
         def rm_func(vios_w):
             LOG.info(_LI("Removing vSCSI mapping from Physical Volume %(dev)s "
                          "to VM %(vm)s") % {'dev': device_name, 'vm': vm_uuid})
-            return tsk_map.remove_maps(
+            removed_maps = tsk_map.remove_maps(
                 vios_w, vm_uuid,
                 tsk_map.gen_match_func(pvm_stor.PV, names=[device_name]))
+            for rm_map in removed_maps:
+                slot_mgr.drop_vscsi_mapping(rm_map)
+            return removed_maps
         self.stg_ftsk.wrapper_tasks[vios_uuid].add_functor_subtask(rm_func)
 
-    def _add_append_mapping(self, vios_uuid, device_name):
-        """This method will update the stg_ftsk to append the mapping to the VIOS
+    def _add_append_mapping(self, vios_uuid, device_name, lpar_slot_num=None,
+                            lua=None):
+        """Update the stg_ftsk to append the mapping to the VIOS.
 
         :param vios_uuid: The UUID of the vios for the pypowervm adapter.
         :param device_name: The The hdisk device name.
+        :param lpar_slot_num: (Optional, Default:None) If specified, the client
+                              lpar slot number to use on the mapping.  If left
+                              as None, it will use the next available slot
+                              number.
+        :param lua: (Optional.  Default: None) Logical Unit Address to set on
+                    the TargetDevice.  If None, the LUA will be assigned by the
+                    server.  Should be specified for all of the VSCSIMappings
+                    for a particular bus, or none of them.
         """
         def add_func(vios_w):
             LOG.info(_LI("Adding vSCSI mapping to Physical Volume %(dev)s "
                          "to VM %(vm)s") % {'dev': device_name,
                                             'vm': self.vm_uuid})
             pv = pvm_stor.PV.bld(self.adapter, device_name)
-            v_map = tsk_map.build_vscsi_mapping(self.host_uuid, vios_w,
-                                                self.vm_uuid, pv)
+            v_map = tsk_map.build_vscsi_mapping(
+                self.host_uuid, vios_w, self.vm_uuid, pv,
+                lpar_slot_num=lpar_slot_num, lua=lua)
             return tsk_map.add_map(vios_w, v_map)
         self.stg_ftsk.wrapper_tasks[vios_uuid].add_functor_subtask(add_func)
 
